#pragma once


#include "cgp/cgp.hpp"
#include "environment.hpp"
#include "key_positions_structure.hpp"
#include "bats.hpp"
#include "dragons.hpp"
#include "projectiles.hpp"
#include "sapin.hpp"


// This definitions allow to use the structures: mesh, mesh_drawable, etc. without mentionning explicitly cgp::
using cgp::mesh;
using cgp::mesh_drawable;
using cgp::vec3;
using cgp::numarray;
using cgp::timer_basic;


// Variables associated to the GUI
struct gui_parameters {
	bool display_frame = true;
	bool display_wireframe = false;
	bool display_hitbox = false;
	float k = 0.5f;
	bool fly = true;
	float speed = 1.0f;
};

// The structure of the custom scene
struct scene_structure : cgp::scene_inputs_generic {
	
	// ****************************** //
	// Elements and shapes of the scene
	// ****************************** //
	camera_controller_first_person_euler camera_control;
	camera_projection_perspective camera_projection;
	window_structure window;

	mesh_drawable global_frame;          // The standard global frame
	environment_structure environment;   // Standard environment controler
	input_devices inputs;                // Storage for inputs status (mouse, keyboard, window dimension)
	gui_parameters gui;                  // Standard GUI element storage
	
	// ****************************** //
	// Elements and shapes of the scene
	// ****************************** //
	
	cgp::skybox_drawable skybox;

	// Timer used for the animation
	timer_basic timer;

	cgp::mesh_drawable terrain;
	cgp::mesh_drawable grass;
	cgp::mesh_drawable crosshair;
	std::vector<cgp::mesh_drawable> menu_icons;
	cgp::mesh_drawable menu_frame;
	int current_menu = 0;
	bool menu_change = false;
	cgp::hierarchy_mesh_drawable healthbar;
	float prev_sim_time = 0;
	float delta_sim_time;
	float ball_turn_speed = 32;
	float ball_reset_speed = 1;
	float smoke_turn_speed = 5;
	float fire_turn_speed = 4;
	cgp::mesh_drawable hitbox_sphere;
	std::vector<image_structure> smoke_grid;
	cgp::mesh_drawable smoke;
	cgp::mesh_drawable fire_breath;
	std::vector<image_structure> fire_breath_grid;
	std::vector<vec3> tree_position;
	std::vector<vec3> grass_position;

	bats bats;
	dragons dragons;
	projectiles projectiles;
	sapin sapin1;

	// ****************************** //
	// Functions
	// ****************************** //

	void initialize();    // Standard initialization to be called before the animation loop
	void display_frame(); // The frame display to be called within the animation loop
	void display_gui();   // The display of the GUI, also called within the animation loop
	void display_grass();
	void display_bats();
	void display_dragons();
	void display_hitbox(hitbox H, vec3 shift, rotation_transform rot);
	void display_healthbar(int hp, int max_hp, vec3 pos);
	void display_death(vec3 pos, double size, int t);
	void display_fire_breath(vec3 pos, rotation_transform rot, float t, float duration);

	void mouse_move_event();
	void mouse_click_event();
	void keyboard_event();
	void idle_frame();

	void display_projectiles();
	void display_crosshair();
	void projectile_menu();

	float day_time();
};


